import numpy as np
from spectramanipulator.spectrum import Spectrum, SpectrumList
from spectramanipulator.parsers.parser import Parser


class GeneralParser(Parser):

    def __init__(self, filepath=None, str_data=None, delimiter='\t', decimal_sep='.', remove_empty_entries=True,
                 skip_col_num=0, general_import_spectra_name_from_filename=False,
                 general_if_header_is_empty_use_filename=True, skip_nan_columns=False, nan_replacement=0):
        super(GeneralParser, self).__init__(filepath, str_data, delimiter, decimal_sep)

        self._spectra_buffer = []
        self.remove_empty_entries = remove_empty_entries
        self.skip_col_num = skip_col_num
        self.general_import_spectra_name_from_filename = general_import_spectra_name_from_filename
        self.general_if_header_is_empty_use_filename = general_if_header_is_empty_use_filename
        self.skip_nan_columns = skip_nan_columns
        self.nan_replacement = nan_replacement

        # history of lines that contained text elements,
        # these are used to extract valuable data (eg. Duetta fluorimeter EEM and kinetics text files)
        self.names_history = []

    def line2list_iterator(self):
        """Iterator method that can be overridden. Get iterator that will iterate through a parsed line into a LIST"""

        # if delimiter is '', split by default - any whitespace and empty strings will be discarded
        delimiter = self.delimiter if self.delimiter != '' else None

        for line in self.__iter__():
            split_line = line.split(delimiter)

            yield split_line

    def parse(self, name=None):

        data = []
        last_names = []

        for line in self.line2list_iterator():
            # line is a parsed list - ['entry1', 'entry2', .... ]

            if len(line) < 2:
                self._parse_chunk(data, last_names)
                continue

            if len(line) != 0:
                # skip defined number of columns
                line = line[self.skip_col_num:]

                # remove empty entries
                if self.remove_empty_entries:
                    line = list(filter(None, line))

            l_values = [self.float_try_parse(num) for num in line]

            if l_values[0] is None:
                self._parse_chunk(data, last_names)
                last_names = line
                self.names_history.append(last_names.copy())

                continue

            data.append(l_values)

        self._parse_chunk(data, last_names)

        return self._spectra_buffer

    def _parse_chunk(self, data, names):
        if len(data) < 2:
            data.clear()
            names.clear()
            return

        col_count = len(data[0])

        if len(names) < len(data[0]):  # resize names of spectra so that len(names) == len(data[0])
            names += [''] * (col_count - len(names))

        # convert parsed data to numpy matrix for easier manipulation
        # by setting a dtype=np.float64, invalid entries will become np.nan
        np_data = np.asarray(data, dtype=np.float64)

        if self.skip_nan_columns:  # if option skip columns containing nan values is true
            nan_cols = np.isnan(np_data).sum(axis=0, keepdims=False)
            value_cols_idxs = np.argwhere(nan_cols == 0).squeeze()
            np_data = np_data[:, value_cols_idxs]
            names = [names[i] for i in value_cols_idxs]   # names is a list, not ndarray
        else:
            np_data = np.nan_to_num(np_data, nan=self.nan_replacement)  # convert nan values to used defined

        # sort according to first column (x values)
        np_data = np_data[np_data[:, 0].argsort()]

        spectra = SpectrumList()
        for i in range(1, np_data.shape[1]):
            # separate the matrix to individual spectra, x values are from first column, the data are from i-th column
            sp_data = np_data[:, [0, i]]

            sp = Spectrum(sp_data, filepath=self.filepath, name=names[i].strip(), assume_sorted=True)
            spectra.children.append(sp)
            spectra.name = self.name_of_file

        # for single spectrum, non-concatenated data
        if len(spectra) == 1:
            name = spectra[0].name
            if self.general_import_spectra_name_from_filename and not self.general_if_header_is_empty_use_filename:
                name = self.name_of_file
            if self.general_if_header_is_empty_use_filename:
                name = self.name_of_file if name == '' else name
            spectra[0].name = name

        if len(spectra) > 0:
            self._spectra_buffer.append(spectra[0] if len(spectra) == 1 else spectra)

        data.clear()
        names.clear()




from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import QMessageBox

from PyQt5.QtCore import Qt
from .gui_interpolate_dialog import Ui_Dialog

from webbrowser import open_new_tab


class InterpolateDialog(QtWidgets.QDialog, Ui_Dialog):

    # static variables
    is_opened = False
    _instance = None

    def __init__(self, parent=None):
        super(InterpolateDialog, self).__init__(parent)
        self.setupUi(self)

        #disable resizing of the window,
        # help from https://stackoverflow.com/questions/16673074/in-qt-c-how-can-i-fully-disable-resizing-a-window-including-the-resize-icon-w
        self.setWindowFlags(Qt.Dialog | Qt.MSWindowsFixedSizeDialogHint)

        self.spacing = 1
        self.selected_kind = 'linear'

        self.setWindowTitle("Interpolate")
        self.leSpacing.setText(str(self.spacing))

        self.kinds = [
            {'kind': 'linear', 'name': 'Linear'},
            {'kind': 'nearest', 'name': 'Nearest'},
            {'kind': 'previous', 'name': 'Previous'},
            {'kind': 'next', 'name': 'Next'},
            {'kind': 'zero', 'name': 'Spline (zero order)'},
            {'kind': 'slinear', 'name': 'Spline (1st order)'},
            {'kind': 'quadratic', 'name': 'Spline (2nd order)'},
            {'kind': 'cubic', 'name': 'Spline (3rd order)'},
        ]

        self.cbIntepolation.addItems(map(lambda d: d['name'], self.kinds))

        self.label.linkActivated.connect(self.open_url)

        self.accepted = False

        InterpolateDialog.is_opened = True
        InterpolateDialog._instance = self

        self.show()
        self.exec()

    def open_url(self):
        open_new_tab('https://docs.scipy.org/doc/scipy/reference/generated/scipy.interpolate.interp1d.html#scipy.interpolate.interp1d')

    @staticmethod
    def get_instance():
        return InterpolateDialog._instance

    def accept(self):
        try:
            self.spacing = float(self.leSpacing.text().replace(',', '.').strip())
        except ValueError:
            QMessageBox.warning(self, 'Error', "Cannot parse spacing value into float, please correct it.", QMessageBox.Ok)
            return

        self.selected_kind = self.kinds[self.cbIntepolation.currentIndex()]['kind']
        self.accepted = True
        InterpolateDialog.is_opened = False
        InterpolateDialog._instance = None
        super(InterpolateDialog, self).accept()

    def reject(self):
        InterpolateDialog.is_opened = False
        InterpolateDialog._instance = None
        super(InterpolateDialog, self).reject()


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = InterpolateDialog()
    # Dialog.show()
    sys.exit(app.exec_())



from pyqtgraph.graphicsItems.ViewBox import ViewBox as _ViewBox

import math


def is_nan_or_inf(value):
    return math.isnan(value) or math.isinf(value)


class ViewBox(_ViewBox):
    # view All fix for downsampling ON

    def __init__(self, plotted_items: dict, **kwargs):
        super(ViewBox, self).__init__(**kwargs)

        self.plotted_items = plotted_items

    def autoRange(self, padding=None, items=None, item=None):
        """Modify the autorange function to properly autorange to all data."""

        spectra = list(self.plotted_items.keys())

        if len(spectra) == 0:
            return

        x0 = spectra[0].data[0, 0]
        x1 = spectra[0].data[-1, 0]

        y0 = spectra[0].y.min()
        y1 = spectra[0].y.max()

        for item in spectra[1:]:
            new_x0 = item.data[0, 0]
            new_x1 = item.data[-1, 0]
            new_y0 = item.y.min()
            new_y1 = item.y.max()

            if new_x0 < x0:
                x0 = new_x0

            if new_x1 > x1:
                x1 = new_x1

            if new_y0 < y0:
                y0 = new_y0

            if new_y1 > y1:
                y1 = new_y1

        x0 = -1 if is_nan_or_inf(x0) else x0
        x1 = 1 if is_nan_or_inf(x1) else x1
        y0 = -1 if is_nan_or_inf(y0) else y0
        y1 = 1 if is_nan_or_inf(y1) else y1

        self.setRange(xRange=(x0, x1), yRange=(y0, y1), padding=padding)









class Singleton(object):
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not isinstance(cls._instance, cls):
            cls._instance = super().__new__(cls, *args, **kwargs)
        return cls._instance

# as meta class
# class Singleton(type):
#     _instances = {}
#
#     def __call__(cls, *args, **kwargs):
#         if cls not in cls._instances:
#             cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
#         return cls._instances[cls]



from PyQt5.QtWidgets import QMenuBar, QAction, QMenu
from spectramanipulator.settings import Settings
from spectramanipulator.dialogs.fitwidget import FitWidget
from spectramanipulator import __version__


class MenuBar(QMenuBar):
    MAX_RECENT_FILES = 20

    def __init__(self, parent=None):
        super(MenuBar, self).__init__(parent=parent)

        # ---File Menu---

        self.file_menu = self.addMenu("&File")

        self.open_project_act = QAction("&Open Project", self)
        self.open_project_act.setShortcut("Ctrl+O")
        self.open_project_act.triggered.connect(self.parent().open_project)
        self.file_menu.addAction(self.open_project_act)

        self.save_project_act = QAction("&Save Project", self)
        self.save_project_act.setShortcut("Ctrl+S")
        self.save_project_act.triggered.connect(self.parent().save_project)
        self.file_menu.addAction(self.save_project_act)

        self.save_project_as_act = QAction("Save Project &As", self)
        self.save_project_as_act.setShortcut("Ctrl+Shift+S")
        self.save_project_as_act.triggered.connect(self.parent().save_project_as)
        self.file_menu.addAction(self.save_project_as_act)

        self.open_recent_menu = QMenu("Open Recent", self.file_menu)
        self.file_menu.addAction(self.open_recent_menu.menuAction())

        # add blank actions
        self.recent_file_actions = []
        for i in range(self.MAX_RECENT_FILES):
            act = self.open_recent_menu.addAction('')
            act.setVisible(False)
            self.recent_file_actions.append(act)
            act.triggered.connect(self.open_recent_file)

        self.file_menu.addSeparator()

        self.import_files_act = QAction("&Import Files", self)
        self.import_files_act.triggered.connect(self.parent().file_menu_import_files)
        # self.import_files.setShortcut("Ctrl+I")
        self.file_menu.addAction(self.import_files_act)

        self.import_special_menu = QMenu("Import Special", self.file_menu)
        self.file_menu.addAction(self.import_special_menu.menuAction())

        self.nano_kinetics = QAction("Kinetics from LFP (with T\u2192A conversion)", self)
        self.nano_kinetics.triggered.connect(self.parent().import_LPF_kinetics)
        self.import_special_menu.addAction(self.nano_kinetics)

        self.EEM_duetta = QAction('Excitation-Emission Map from Duetta Fluorimeter', self)
        self.EEM_duetta.triggered.connect(self.parent().import_EEM_Duetta)
        self.import_special_menu.addAction(self.EEM_duetta)

        self.kinetics_duetta = QAction('Kinetics from Duetta Fluorimeter', self)
        self.kinetics_duetta.triggered.connect(self.parent().import_kinetics_Duetta)
        self.import_special_menu.addAction(self.kinetics_duetta)

        self.batch_load_kin = QAction('Batch Load UV-VIS kinetics', self)
        self.batch_load_kin.triggered.connect(self.parent().batch_load_kinetics)
        self.import_special_menu.addAction(self.batch_load_kin)

        self.export_selected_spectra_as_act = QAction("&Export Selected Items As", self)
        # self.export_selected_spectra_as.setShortcut("Ctrl+E")
        self.export_selected_spectra_as_act.triggered.connect(self.parent().tree_widget.export_selected_items_as)
        self.file_menu.addAction(self.export_selected_spectra_as_act)

        self.file_menu.addSeparator()

        self.settings_act = QAction("Se&ttings", self)
        self.settings_act.triggered.connect(self.parent().open_settings)
        self.file_menu.addAction(self.settings_act)

        self.file_menu.addSeparator()

        self.exit_act = QAction("E&xit", self)
        self.exit_act.triggered.connect(self.parent().close)
        self.file_menu.addAction(self.exit_act)

        # ---Utilities---

        self.utilities_plot_menu = self.addMenu("&Utilities")

        self.function_plotter_act = QAction("&Function plotter", self)
        self.function_plotter_act.triggered.connect(self.open_function_plotter)
        self.utilities_plot_menu.addAction(self.function_plotter_act)

        # ---Export Plot----

        # self.export_plot_menu = self.addMenu("&Export Plot")
        #
        # self.copy_as_image_act = QAction("&Copy As Image", self)
        # self.copy_as_image_act.triggered.connect(self.parent().grpView.save_plot_to_clipboard_as_png)
        # self.export_plot_menu.addAction(self.copy_as_image_act)
        #
        # self.copy_as_svg = QAction("&Copy As SVG", self)
        # self.copy_as_svg.triggered.connect(self.parent().grpView.save_plot_to_clipboard_as_svg)
        # self.export_plot_menu.addAction(self.copy_as_svg)

        # ---About Menu---

        self.about_menu = self.addMenu("&Help")

        self.about_act = QAction("&About", self)
        self.about_act.triggered.connect(self.show_about_window)
        self.about_menu.addAction(self.about_act)

        self.update_act = QAction("&Check for updates...", self)
        self.update_act.triggered.connect(self.parent().check_for_updates)
        self.about_menu.addAction(self.update_act)

    def open_function_plotter(self):
        # TODO
        return
        # def accepted():
        #     self.parent().tree_widget.import_spectra(fit_widget.plotted_function_spectra)
        #     self.parent().tree_widget.state_changed.emit()
        #
        # fit_widget = FitWidget(self.parent().var_widget, accepted, None, parent=self)

    def open_recent_file(self):
        if self.sender():
            self.parent().open_project(filepath=self.sender().data(), open_dialog=False)

    def show_about_window(self):
        # self.parent().console.setVisible(True)
        if not self.parent().console_button.isChecked():
            self.parent().console_button.toggle()

        about_message = f"""
             
<h3>Simple Spectra Manipulator</h3>
<p>Version:&nbsp;<span style="color: #ff00ff;"><strong>{__version__}</strong></span></p>
<p><span style="color: #000000;">This software is distributed under the <span style="color: #339966;"><strong>MIT open-source licence&nbsp;</strong></span></span></p>
<p>Copyright (c) 2020 Dominik Madea</p>
<p>Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:</p>
<p>The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.</p>
<p>THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.</p>"""

        self.parent().console.print_html(about_message)




from PyQt5 import QtCore, QtGui, QtWidgets

# from PyQt5 import *
from PyQt5.QtCore import Qt
# from PyQt5.QtWidgets import *
from .gui_intintinputdialog import Ui_Dialog
import sys


class IntIntInputDialog(QtWidgets.QDialog, Ui_Dialog):

    # static variables
    is_opened = False
    _instance = None

    int32_max = 2147483647

    def __init__(self, n=1, offset=0, n_min=1, offset_min=0, title='Int Int Input Dialog', label='Set xrange',  parent=None):
        super(IntIntInputDialog, self).__init__(parent)
        self.setupUi(self)

        #disable resizing of the window,
        # help from https://stackoverflow.com/questions/16673074/in-qt-c-how-can-i-fully-disable-resizing-a-window-including-the-resize-icon-w
        self.setWindowFlags(Qt.Dialog | Qt.MSWindowsFixedSizeDialogHint)

        self.setWindowTitle(title)
        self.label.setText(label)

        # self.dbX0.valueChanged.connect(self.dbX0_value_changed)
        # self.dbX1.valueChanged.connect(self.dbX1_value_changed)

        self.sbn.setValue(n)
        self.sbOffset.setValue(offset)

        self.sbn.setMinimum(n_min)
        self.sbOffset.setMinimum(offset_min)

        self.sbn.setMaximum(self.int32_max)
        self.sbOffset.setMaximum(self.int32_max)

        self.accepted = False

        IntIntInputDialog.is_opened = True
        IntIntInputDialog._instance = self

        self.sbn.setFocus()
        self.sbn.selectAll()

        self.show()
        self.exec()

    @staticmethod
    def get_instance():
        return IntIntInputDialog._instance

    def set_result(self):
        self.returned_range = (self.sbn.value(), self.sbOffset.value())

    def accept(self):
        self.set_result()
        self.accepted = True
        IntIntInputDialog.is_opened = False
        IntIntInputDialog._instance = None
        super(IntIntInputDialog, self).accept()

    def reject(self):
        IntIntInputDialog.is_opened = False
        IntIntInputDialog._instance = None
        super(IntIntInputDialog, self).reject()


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = IntIntInputDialog()
    # Dialog.show()
    sys.exit(app.exec_())


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'spectramanipulator/dialogs\gui_load_kinetics.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(475, 493)
        self.gridLayout_2 = QtWidgets.QGridLayout(Dialog)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_4 = QtWidgets.QLabel(Dialog)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        self.btnChooseDirs = QtWidgets.QToolButton(Dialog)
        self.btnChooseDirs.setObjectName("btnChooseDirs")
        self.horizontalLayout_3.addWidget(self.btnChooseDirs)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.lwGridLayout = QtWidgets.QGridLayout()
        self.lwGridLayout.setObjectName("lwGridLayout")
        self.verticalLayout.addLayout(self.lwGridLayout)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.leSpectra = QtWidgets.QLineEdit(Dialog)
        self.leSpectra.setObjectName("leSpectra")
        self.gridLayout.addWidget(self.leSpectra, 1, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(Dialog)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.leBlank = QtWidgets.QLineEdit(Dialog)
        self.leBlank.setObjectName("leBlank")
        self.gridLayout.addWidget(self.leBlank, 2, 1, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 7, 0, 1, 2)
        self.leTimes = QtWidgets.QLineEdit(Dialog)
        self.leTimes.setObjectName("leTimes")
        self.gridLayout.addWidget(self.leTimes, 4, 1, 1, 1)
        self.cbCut = QtWidgets.QCheckBox(Dialog)
        self.cbCut.setChecked(False)
        self.cbCut.setObjectName("cbCut")
        self.gridLayout.addWidget(self.cbCut, 6, 0, 1, 1)
        self.cbBCorr = QtWidgets.QCheckBox(Dialog)
        self.cbBCorr.setChecked(False)
        self.cbBCorr.setObjectName("cbBCorr")
        self.gridLayout.addWidget(self.cbBCorr, 5, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(Dialog)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 4, 0, 1, 1)
        self.leTimeUnit = QtWidgets.QLineEdit(Dialog)
        self.leTimeUnit.setObjectName("leTimeUnit")
        self.gridLayout.addWidget(self.leTimeUnit, 3, 1, 1, 1)
        self.cbKineticsMeasuredByEach = QtWidgets.QCheckBox(Dialog)
        self.cbKineticsMeasuredByEach.setChecked(True)
        self.cbKineticsMeasuredByEach.setObjectName("cbKineticsMeasuredByEach")
        self.gridLayout.addWidget(self.cbKineticsMeasuredByEach, 3, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.leBCorr0 = QtWidgets.QLineEdit(Dialog)
        self.leBCorr0.setObjectName("leBCorr0")
        self.horizontalLayout.addWidget(self.leBCorr0)
        self.label_6 = QtWidgets.QLabel(Dialog)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout.addWidget(self.label_6)
        self.leBCorr1 = QtWidgets.QLineEdit(Dialog)
        self.leBCorr1.setObjectName("leBCorr1")
        self.horizontalLayout.addWidget(self.leBCorr1)
        self.gridLayout.addLayout(self.horizontalLayout, 5, 1, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.leCut0 = QtWidgets.QLineEdit(Dialog)
        self.leCut0.setObjectName("leCut0")
        self.horizontalLayout_2.addWidget(self.leCut0)
        self.label_7 = QtWidgets.QLabel(Dialog)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_2.addWidget(self.label_7)
        self.leCut1 = QtWidgets.QLineEdit(Dialog)
        self.leCut1.setObjectName("leCut1")
        self.horizontalLayout_2.addWidget(self.leCut1)
        self.gridLayout.addLayout(self.horizontalLayout_2, 6, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label.setText(_translate("Dialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'MS Shell Dlg 2\'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Loads UV-Vis kinetics. The spectra can be in various formats (dx, csv, txt, etc.). If blank spectrum is provided, it will be subtracted from each of the spectra. Time corresponding to each spectrum will be set as the name for that spectrum. Times can be provided by time difference (in case kinetics was measured reguraly) or from filename. Postprocessing (baseline correction and/or cut of spectra) can be performed on loaded dataset.</p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Required folder structure for one kinetics:</p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">[kinetics folder]<span style=\" font-family:\'JetBrains Mono,monospace\'; color:#629755;\"><br />    </span><span style=\" font-family:\'JetBrains Mono,monospace\'; color:#000000;\">[spectra folder]<br />         01.dx<br />         02.dx<br />         ...<br />    times.txt (optional)<br />    blank.dx (optional)</span></p></body></html>"))
        self.label_4.setText(_translate("Dialog", "Folders to load:"))
        self.btnChooseDirs.setText(_translate("Dialog", "..."))
        self.label_2.setText(_translate("Dialog", "Blank spectrum name (optional):"))
        self.leSpectra.setText(_translate("Dialog", "spectra"))
        self.label_3.setText(_translate("Dialog", "Spectra folder name:"))
        self.leBlank.setText(_translate("Dialog", "blank.dx"))
        self.leTimes.setText(_translate("Dialog", "times.txt"))
        self.cbCut.setText(_translate("Dialog", "Cut spectra to range:"))
        self.cbBCorr.setText(_translate("Dialog", "Apply baseline correction in range:"))
        self.label_5.setText(_translate("Dialog", "Use times from filename (optional):"))
        self.leTimeUnit.setText(_translate("Dialog", "1"))
        self.cbKineticsMeasuredByEach.setText(_translate("Dialog", "Kinetics measured by each (time unit):"))
        self.leBCorr0.setText(_translate("Dialog", "700"))
        self.label_6.setText(_translate("Dialog", "to"))
        self.leBCorr1.setText(_translate("Dialog", "800"))
        self.leCut0.setText(_translate("Dialog", "230"))
        self.label_7.setText(_translate("Dialog", "to"))
        self.leCut1.setText(_translate("Dialog", "650"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())



from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import QMessageBox

# from PyQt5 import *
from PyQt5.QtCore import Qt
# from PyQt5.QtWidgets import *
from .gui_settings import Ui_Dialog

from spectramanipulator.settings import Settings
# from logger import Logger


class SettingsDialog(QtWidgets.QDialog, Ui_Dialog):
    # static variables
    is_opened = False
    _instance = None

    def __init__(self, parent=None):
        super(SettingsDialog, self).__init__(parent, QtCore.Qt.WindowStaysOnTopHint)
        self.setupUi(self)
        self.setWindowTitle("Settings")

        self.tabWidget.setCurrentIndex(Settings.gui_settings_last_tab_index)
        self.dx_if_title_is_empty_use_filename.stateChanged.connect(self.dx_if_title_is_empty_use_filename_checked_changed)
        self.general_if_header_is_empty_use_filename.stateChanged.connect(self.general_if_header_is_empty_use_filename_checked_changed)

        self.rbDefaultColorScheme.toggled.connect(self.schemes_changes)
        self.rbHSVColorScheme.toggled.connect(self.schemes_changes)
        self.rbUserColorScheme.toggled.connect(self.schemes_changes)
        self.cbSkipNaNColumns.toggled.connect(self.cbSkipNaNColumns_toggled)

        self.btnRestoreDefaultSettings.clicked.connect(self.restore_default_settings)

        self.load_settings()

        SettingsDialog.is_opened = True
        SettingsDialog._instance = self

        self.show()
        self.exec()

    def cbSkipNaNColumns_toggled(self):
        self.sbNaNReplacement.setEnabled(not self.cbSkipNaNColumns.isChecked())

    def restore_default_settings(self):
        Settings.set_default_settings()
        self.load_settings()

    # def rbSkipColumns_changed(self):
    #     if self.rbSkipColumns.isChecked():
    #         self.sbColumns.setEnabled(True)
    #     else:
    #         self.sbColumns.setEnabled(False)

    def schemes_changes(self):
        if self.rbDefaultColorScheme.isChecked():
            self.sbHues.setEnabled(False)
            self.sbValues.setEnabled(False)
            self.sbMinHue.setEnabled(False)
            self.sbMaxHue.setEnabled(False)
            self.sbMinValue.setEnabled(False)
            self.sbMaxValue.setEnabled(False)
            self.txbUserColorScheme.setEnabled(False)

        if self.rbHSVColorScheme.isChecked():
            self.sbHues.setEnabled(True)
            self.sbValues.setEnabled(True)
            self.sbMinHue.setEnabled(True)
            self.sbMaxHue.setEnabled(True)
            self.sbMinValue.setEnabled(True)
            self.sbMaxValue.setEnabled(True)
            self.txbUserColorScheme.setEnabled(False)

        if self.rbUserColorScheme.isChecked():
            self.sbHues.setEnabled(True)
            self.sbValues.setEnabled(False)
            self.sbMinHue.setEnabled(False)
            self.sbMaxHue.setEnabled(False)
            self.sbMinValue.setEnabled(False)
            self.sbMaxValue.setEnabled(False)
            self.txbUserColorScheme.setEnabled(True)


    # def rbDefaultColorScheme_checked_changed(self):

    def dx_if_title_is_empty_use_filename_checked_changed(self):
        if self.dx_if_title_is_empty_use_filename.checkState() == Qt.Checked:
            self.dx_import_spectra_name_from_filename.setEnabled(False)
            self.rb_DX_import_from_title.setEnabled(False)
        else:
            self.dx_import_spectra_name_from_filename.setEnabled(True)
            self.rb_DX_import_from_title.setEnabled(True)

    def general_if_header_is_empty_use_filename_checked_changed(self):
        if self.general_if_header_is_empty_use_filename.checkState() == Qt.Checked:
            self.general_import_spectra_name_from_filename.setEnabled(False)
            self.rb_general_import_from_header.setEnabled(False)
        else:
            self.general_import_spectra_name_from_filename.setEnabled(True)
            self.rb_general_import_from_header.setEnabled(True)

    @staticmethod
    def check_state(checked):
        return Qt.Checked if checked else Qt.Unchecked

    @staticmethod
    def DEcheck_state(checked):
        return True if checked == Qt.Checked else False

    @staticmethod
    def textualize_special_chars(text):
        return text.replace('\n', '\\n').replace('\t', '\\t').replace('\r', '\\r')

    @staticmethod
    def DEtextualize_special_chars(text):
        return text.replace('\\n', '\n').replace('\\t', '\t').replace('\\r', '\r')

    def load_settings(self):
        # import options

        self.cbRemoveEmptyEntries.setCheckState(self.check_state(Settings.remove_empty_entries))
        self.sbColumns.setValue(Settings.skip_columns_num)

        self.csv_imp_delimiter.setText(self.textualize_special_chars(Settings.csv_imp_delimiter))
        self.csv_imp_decimal_sep.setText(self.textualize_special_chars(Settings.csv_imp_decimal_sep))
        self.general_imp_delimiter.setText(self.textualize_special_chars(Settings.general_imp_delimiter))
        self.general_imp_decimal_sep.setText(self.textualize_special_chars(Settings.general_imp_decimal_sep))
        self.dx_imp_delimiter.setText(self.textualize_special_chars(Settings.dx_imp_delimiter))
        self.dx_imp_decimal_sep.setText(self.textualize_special_chars(Settings.dx_imp_decimal_sep))

        self.dx_import_spectra_name_from_filename.setChecked(Settings.dx_import_spectra_name_from_filename)
        self.rb_DX_import_from_title.setChecked(not Settings.dx_import_spectra_name_from_filename)
        self.dx_if_title_is_empty_use_filename.setCheckState(self.check_state(Settings.dx_if_title_is_empty_use_filename))

        self.general_import_spectra_name_from_filename.setChecked(Settings.general_import_spectra_name_from_filename)
        self.rb_general_import_from_header.setChecked(not Settings.general_import_spectra_name_from_filename)
        self.general_if_header_is_empty_use_filename.setCheckState(
            self.check_state(Settings.general_if_header_is_empty_use_filename))

        self.cbSkipNaNColumns.setChecked(Settings.skip_nan_columns)
        self.sbNaNReplacement.setValue(Settings.nan_replacement)

        # perform change
        self.dx_if_title_is_empty_use_filename_checked_changed()

        self.clip_imp_delimiter.setText(self.textualize_special_chars(Settings.clip_imp_delimiter))
        self.clip_imp_decimal_sep.setText(self.textualize_special_chars(Settings.clip_imp_decimal_sep))

        self.excel_imp_as_text.setCheckState(self.check_state(Settings.excel_imp_as_text))

        # file export options

        self.files_exp_include_group_name.setCheckState(self.check_state(Settings.files_exp_include_group_name))
        self.files_exp_include_header.setCheckState(self.check_state(Settings.files_exp_include_header))
        # self.csv_exp_delimiter.setText(self.textualize_special_chars(Settings.csv_exp_delimiter))
        # self.csv_exp_decimal_sep.setText(self.textualize_special_chars(Settings.csv_exp_decimal_sep))
        # self.general_exp_delimiter.setText(self.textualize_special_chars(Settings.general_exp_delimiter))
        # self.general_exp_decimal_sep.setText(self.textualize_special_chars(Settings.general_exp_decimal_sep))

        # clipboard export options

        self.clip_exp_include_group_name.setCheckState(self.check_state(Settings.clip_exp_include_group_name))
        self.clip_exp_include_header.setCheckState(self.check_state(Settings.clip_exp_include_header))
        self.clip_exp_delimiter.setText(self.textualize_special_chars(Settings.clip_exp_delimiter))
        self.clip_exp_decimal_sep.setText(self.textualize_special_chars(Settings.clip_exp_decimal_sep))

        # plotting settings

        self.graph_title.setText(Settings.graph_title)
        self.antialiasing.setCheckState(self.check_state(Settings.antialiasing))
        self.left_axis_label.setText(Settings.left_axis_label)
        # self.left_axis_unit = None

        self.bottom_axis_label.setText(Settings.bottom_axis_label)
        # self.bottom_axis_unit = None

        self.show_grid.setCheckState(self.check_state(Settings.show_grid))
        self.grid_alpha.setValue(Settings.grid_alpha)

        self.line_width.setValue(Settings.line_width)

        self.same_color_in_group.setCheckState(self.check_state(Settings.same_color_in_group))
        self.different_line_style_among_groups.setCheckState(self.check_state(Settings.different_line_style_among_groups))

        self.legend_spacing.setValue(Settings.legend_spacing)

        if Settings.color_scheme == 0:
            self.rbDefaultColorScheme.setChecked(True)
        elif Settings.color_scheme == 1:
            self.rbHSVColorScheme.setChecked(True)
        else:
            self.rbUserColorScheme.setChecked(True)

        self.cbHSVReversed.setChecked(Settings.HSV_reversed)

        self.sbHues.setValue(Settings.hues)
        self.sbValues.setValue(Settings.values)
        self.sbMinHue.setValue(Settings.minHue)
        self.sbMaxHue.setValue(Settings.maxHue)
        self.sbMinValue.setValue(Settings.minValue)
        self.sbMaxValue.setValue(Settings.maxValue)

        self.txbUserColorScheme.setPlainText(Settings.user_defined_grad)
        self.chbReverseZOrder.setChecked(self.check_state(Settings.reverse_z_order))

    def save_settings(self):
        if self.csv_imp_decimal_sep.text() == "" or self.general_imp_decimal_sep.text() == "" or \
                self.dx_imp_decimal_sep.text() == "" or self.clip_imp_decimal_sep.text() == "" or \
                self.clip_exp_decimal_sep.text() == "" or self.clip_exp_delimiter == "":
            raise ValueError("Please check decimal separator fields, these cannot be empty.")

        # import options

        Settings.remove_empty_entries = self.DEcheck_state(self.cbRemoveEmptyEntries.checkState())
        Settings.skip_columns_num = int(self.sbColumns.value())

        Settings.csv_imp_delimiter = self.DEtextualize_special_chars(self.csv_imp_delimiter.text())
        Settings.csv_imp_decimal_sep = self.DEtextualize_special_chars(self.csv_imp_decimal_sep.text())
        Settings.general_imp_delimiter = self.DEtextualize_special_chars(self.general_imp_delimiter.text())
        Settings.general_imp_decimal_sep = self.DEtextualize_special_chars(self.general_imp_decimal_sep.text())
        Settings.dx_imp_delimiter = self.DEtextualize_special_chars(self.dx_imp_delimiter.text())
        Settings.dx_imp_decimal_sep = self.DEtextualize_special_chars(self.dx_imp_decimal_sep.text())

        Settings.dx_import_spectra_name_from_filename = self.dx_import_spectra_name_from_filename.isChecked()
        Settings.dx_if_title_is_empty_use_filename = self.DEcheck_state(self.dx_if_title_is_empty_use_filename.checkState())

        Settings.general_import_spectra_name_from_filename = self.general_import_spectra_name_from_filename.isChecked()
        Settings.general_if_header_is_empty_use_filename = self.DEcheck_state(
            self.general_if_header_is_empty_use_filename.checkState())

        Settings.skip_nan_columns = self.cbSkipNaNColumns.isChecked()
        Settings.nan_replacement = float(self.sbNaNReplacement.value())

        Settings.clip_imp_delimiter = self.DEtextualize_special_chars(self.clip_imp_delimiter.text())
        Settings.clip_imp_decimal_sep = self.DEtextualize_special_chars(self.clip_imp_decimal_sep.text())

        Settings.excel_imp_as_text = self.DEcheck_state(self.excel_imp_as_text.checkState())

        # file export options

        Settings.files_exp_include_group_name = self.DEcheck_state(self.files_exp_include_group_name.checkState())
        Settings.files_exp_include_header = self.DEcheck_state(self.files_exp_include_header.checkState())
        # Settings.csv_exp_delimiter = self.DEtextualize_special_chars(self.csv_exp_delimiter.text())
        # Settings.csv_exp_decimal_sep = self.DEtextualize_special_chars(self.csv_exp_decimal_sep.text())
        # Settings.general_exp_delimiter = self.DEtextualize_special_chars(self.general_exp_delimiter.text())
        # Settings.general_exp_decimal_sep = self.DEtextualize_special_chars(self.general_exp_decimal_sep.text())

        # clipboard export options

        Settings.clip_exp_include_group_name = self.DEcheck_state(self.clip_exp_include_group_name.checkState())
        Settings.clip_exp_include_header = self.DEcheck_state(self.clip_exp_include_header.checkState())
        Settings.clip_exp_delimiter = self.DEtextualize_special_chars(self.clip_exp_delimiter.text())
        Settings.clip_exp_decimal_sep = self.DEtextualize_special_chars(self.clip_exp_decimal_sep.text())

        # plotting settings

        Settings.graph_title = self.graph_title.text()
        Settings.antialiasing = self.DEcheck_state(self.antialiasing.checkState())
        Settings.left_axis_label = self.left_axis_label.text()
        # Settings.left_axis_unit = None

        Settings.bottom_axis_label = self.bottom_axis_label.text()
        # bottom_axis_unit = None

        Settings.show_grid = self.DEcheck_state(self.show_grid.checkState())
        Settings.grid_alpha = self.grid_alpha.value()

        Settings.line_width = self.line_width.value()

        Settings.same_color_in_group = self.DEcheck_state(self.same_color_in_group.checkState())
        Settings.different_line_style_among_groups = self.DEcheck_state(self.different_line_style_among_groups.checkState())

        Settings.legend_spacing = self.legend_spacing.value()

        # Settings.HSV_color_scheme = self.rbHSVColorScheme.isChecked()
        if self.rbDefaultColorScheme.isChecked():
            Settings.color_scheme = 0
        elif self.rbHSVColorScheme.isChecked():
            Settings.color_scheme = 1
        else:
            Settings.color_scheme = 2

        Settings.hues = self.sbHues.value()
        Settings.values = self.sbValues.value()
        Settings.minHue = self.sbMinHue.value()
        Settings.maxHue = self.sbMaxHue.value()
        Settings.minValue = self.sbMinValue.value()
        Settings.maxValue = self.sbMaxValue.value()
        Settings.HSV_reversed = self.cbHSVReversed.isChecked()

        Settings.user_defined_grad = self.txbUserColorScheme.toPlainText()
        Settings.reverse_z_order = self.DEcheck_state(self.chbReverseZOrder.checkState())



    @staticmethod
    def get_instance():
        return SettingsDialog._instance

    def accept(self):
        try:
            self.save_settings()
        except ValueError as ex:
            QMessageBox.warning(self, 'Information', ex.__str__(), QMessageBox.Ok)
            return

        Settings.gui_settings_last_tab_index = self.tabWidget.currentIndex()
        Settings.save()

        self.accepted = True
        SettingsDialog.is_opened = False
        SettingsDialog._instance = None
        super(SettingsDialog, self).accept()

    def reject(self):
        SettingsDialog.is_opened = False
        SettingsDialog._instance = None
        super(SettingsDialog, self).reject()


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Dialog = SettingsDialog()
    # Dialog.show()
    sys.exit(app.exec_())



import json
from spectramanipulator.logger import Logger
# import sys
import os


class Settings(object):
    """Static class containing user settings with static functions for saving and loading to/from json file."""

    # settings that starts with '_' will not be saved into a file
    # static variables

    _config_filename = "config.json"
    _inst_default_settings = None  # default settings instance

    # debug = False

    # not in Settings dialog
    enable_multiprocessing = True  # enables importing files as separate processes
    force_multiprocess = False  # importing of files will only run in multiple processes

    # --------SETTINGS----------

    # not in Settings dialog
    general_models_dir = 'general models'
    REG_PROGRAM_NAME = 'SpectraManipulator.projectfile'
    PROJECT_EXTENSION = '.smpj'

    # import options

    remove_empty_entries = True
    skip_columns_num = 0
    skip_nan_columns = False
    nan_replacement = 0

    csv_imp_delimiter = ','
    csv_imp_decimal_sep = '.'
    general_imp_delimiter = ''
    general_imp_decimal_sep = '.'
    dx_imp_delimiter = ' '
    dx_imp_decimal_sep = '.'
    dx_import_spectra_name_from_filename = False
    dx_if_title_is_empty_use_filename = True
    general_import_spectra_name_from_filename = False
    general_if_header_is_empty_use_filename = True

    clip_imp_delimiter = '\t'
    clip_imp_decimal_sep = '.'

    excel_imp_as_text = False

    # file export options

    files_exp_include_group_name = False
    files_exp_include_header = True

    # clipboard export options

    clip_exp_include_group_name = True
    clip_exp_include_header = True
    clip_exp_delimiter = '\t'
    clip_exp_decimal_sep = '.'

    # plotting settings

    graph_title = ""

    antialiasing = True
    left_axis_label = "Absorbance"
    left_axis_unit = None

    bottom_axis_label = "Wavelength / nm"
    bottom_axis_unit = None

    show_grid = False
    grid_alpha = 0.1

    line_width = 1

    graph_title_font_size = 20
    bottom_axis_font_size = 20
    left_axis_font_size = 20

    same_color_in_group = False
    different_line_style_among_groups = False

    legend_spacing = 8

    color_scheme = 0  # 0 - default, 1 - HSV, 2 - user defined

    # plotting of fits

    fit_dark_factor = 0.7

    # plotwidget settings
    # not in Settings dialog
    coordinates_sig_figures = 4  # number of significant figures displayed in status bar for cooridnates

    # HSV_color_scheme = True
    hues = 9
    values = 1
    maxValue = 255
    minValue = 150
    maxHue = 360
    minHue = 0
    sat = 255  # saturation
    alpha = 255  # alpha

    user_defined_grad = "0.0\t1\t0\t0\t1\n0.5\t0\t1\t0\t1\n1.0\t0\t0\t1\t1"
    HSV_reversed = False  # reverse colors

    reverse_z_order = False

    # function plotter

    FP_num_of_points = 500  # number of points to be plotted on selected range for function plotter

    # rename dialog

    last_rename_expression = ""
    last_rename_take_name_from_list = False

    # files dialog last paths

    import_files_dialog_path = ""
    import_LPF_dialog_path = ""
    import_EEM_dialog_path = ""
    open_project_dialog_path = ""
    save_project_dialog_path = ""
    export_spectra_as_dialog_path = ""
    export_spectra_as_dialog_ext = '.txt'
    export_spectra_as_dialog_delimiter = '\t'
    export_spectra_as_dialog_decimal_sep = '.'

    load_kinetics_last_path = ""

    # recent projects filepaths

    recent_project_filepaths = []

    gui_settings_last_tab_index = 0

    _project_independent_settings = [
        'export_spectra_as_dialog_path',
        'export_spectra_as_dialog_ext',
        'export_spectra_as_dialog_delimiter',
        'export_spectra_as_dialog_decimal_sep',
        'recent_project_filepaths',
        'import_files_dialog_path',
        'import_LPF_dialog_path',
        'import_EEM_dialog_path',
        'open_project_dialog_path',
        'save_project_dialog_path',
        'legend_spacing',
        'line_width'
    ]

    def __init__(self):
        """Saves the static attributes to attr and removes project independent settings."""
        self.attr = Settings.get_attributes()

        # delete settings that are project independent
        for s in Settings._project_independent_settings:
            del self.attr[s]

    def set_settings(self):
        """Sets static settings from this instance object (project settings)."""
        # dont set any settings
        pass

        # for key, value in self.attr.items():
        #     setattr(Settings, key, value)

        del self

    @classmethod
    def set_default_settings(cls):
        if Settings._inst_default_settings is not None:
            Settings._inst_default_settings.set_settings()

    @classmethod
    def get_attributes(cls):
        members = vars(cls)
        filtered = {attr: key for attr, key in members.items() if not attr.startswith('_') and
                    not callable(getattr(Settings, attr))}
        return filtered

    @classmethod
    def get_config_filepath(cls):
        curr_dir = os.path.dirname(os.path.realpath(__file__))
        return os.path.join(curr_dir, cls._config_filename)

    @classmethod
    def save(cls):

        sett_dict = cls.get_attributes()

        try:

            with open(cls.get_config_filepath(), "w") as file:
                json.dump(sett_dict, file, sort_keys=False, indent=4, separators=(',', ': '))

        except Exception as ex:
            Logger.message("Error saving settings to file {}. Error message:\n{}".format(
                Settings._config_filename, ex.__str__()))

    @classmethod
    def load(cls):
        # basically static constructor method, set up default settings instance
        if Settings._inst_default_settings is None:
            Settings._inst_default_settings = Settings()

        try:
            with open(cls.get_config_filepath(), "r") as file:
                data = json.load(file)

            # instance = object.__new__(cls)

            for key, value in data.items():
                setattr(cls, key, value)

        except Exception as ex:
            Logger.message(
                "Error loading settings from file {}, setting up default settings. Error message:\n{}".format(
                    Settings._config_filename, ex.__str__()))
            Settings.save()

# Settings.save()
#
# inst = Settings()
#
# # inst.save_project_dialog_path = "asaps pasdasdpa sdpaosapsdo apsdopaso pasokd aosdkaps paskdpaoskdpaoskdpaos"
#
# # inst.attr['normalize_range'] = (0, 0)
#
# inst.set_settings()
#
# print(Settings._get_attributes())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'spectramanipulator/dialogs\gui_rename_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(421, 324)
        self.gridLayout_2 = QtWidgets.QGridLayout(Dialog)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(Dialog)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(Dialog)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 2, 0, 1, 1)
        self.leExpression = QtWidgets.QLineEdit(Dialog)
        self.leExpression.setObjectName("leExpression")
        self.gridLayout.addWidget(self.leExpression, 0, 1, 1, 1)
        self.sbOffset = QtWidgets.QSpinBox(Dialog)
        self.sbOffset.setObjectName("sbOffset")
        self.gridLayout.addWidget(self.sbOffset, 1, 1, 1, 1)
        self.leCounterMulFactor = QtWidgets.QLineEdit(Dialog)
        self.leCounterMulFactor.setObjectName("leCounterMulFactor")
        self.gridLayout.addWidget(self.leCounterMulFactor, 2, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.cbTakeNamesFromList = QtWidgets.QCheckBox(Dialog)
        self.cbTakeNamesFromList.setObjectName("cbTakeNamesFromList")
        self.verticalLayout_2.addWidget(self.cbTakeNamesFromList)
        self.leList = QtWidgets.QLineEdit(Dialog)
        self.leList.setObjectName("leList")
        self.verticalLayout_2.addWidget(self.leList)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label.setText(_translate("Dialog", "<html><head/><body><p>Renames selected items. {:2d} - integer counter, {:2f} - float counter, {:2g} - significant digits counter. for more see https://pyformat.info/. For original name slicing, use {start_idx:end_idx}, same from python slicing rules. Eg. expression is \'{:02d}: t = {:} us\', current name is \'167\' and current counter is 16. Resulting name will be \'16: t = 167 us\'. Current counter (integer counter starts with counter offset value) for each item is multiplied by counter mult. factor.</p></body></html>"))
        self.label_2.setText(_translate("Dialog", "Expression:"))
        self.label_3.setText(_translate("Dialog", "Counter offset:"))
        self.label_4.setText(_translate("Dialog", "Counter mult. factor:"))
        self.leCounterMulFactor.setText(_translate("Dialog", "1"))
        self.cbTakeNamesFromList.setText(_translate("Dialog", "Take names from list (separate values by comma):"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'spectramanipulator/dialogs\gui_settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.setEnabled(True)
        Dialog.resize(697, 800)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Dialog.sizePolicy().hasHeightForWidth())
        Dialog.setSizePolicy(sizePolicy)
        Dialog.setMinimumSize(QtCore.QSize(0, 0))
        Dialog.setMaximumSize(QtCore.QSize(999999, 999999))
        self.gridLayout_5 = QtWidgets.QGridLayout(Dialog)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(Dialog)
        self.tabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.tabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabWidget.setElideMode(QtCore.Qt.ElideNone)
        self.tabWidget.setUsesScrollButtons(True)
        self.tabWidget.setDocumentMode(False)
        self.tabWidget.setTabsClosable(False)
        self.tabWidget.setMovable(False)
        self.tabWidget.setTabBarAutoHide(True)
        self.tabWidget.setObjectName("tabWidget")
        self.tImport = QtWidgets.QWidget()
        self.tImport.setObjectName("tImport")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tImport)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_11 = QtWidgets.QLabel(self.tImport)
        self.label_11.setWordWrap(True)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_2.addWidget(self.label_11)
        self.groupBox_7 = QtWidgets.QGroupBox(self.tImport)
        self.groupBox_7.setObjectName("groupBox_7")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox_7)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.cbRemoveEmptyEntries = QtWidgets.QCheckBox(self.groupBox_7)
        self.cbRemoveEmptyEntries.setObjectName("cbRemoveEmptyEntries")
        self.verticalLayout_10.addWidget(self.cbRemoveEmptyEntries)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_12 = QtWidgets.QLabel(self.groupBox_7)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_3.addWidget(self.label_12)
        self.sbColumns = QtWidgets.QSpinBox(self.groupBox_7)
        self.sbColumns.setMaximum(9999999)
        self.sbColumns.setObjectName("sbColumns")
        self.horizontalLayout_3.addWidget(self.sbColumns)
        self.verticalLayout_10.addLayout(self.horizontalLayout_3)
        self.cbSkipNaNColumns = QtWidgets.QCheckBox(self.groupBox_7)
        self.cbSkipNaNColumns.setObjectName("cbSkipNaNColumns")
        self.verticalLayout_10.addWidget(self.cbSkipNaNColumns)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_13 = QtWidgets.QLabel(self.groupBox_7)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_5.addWidget(self.label_13)
        self.sbNaNReplacement = QtWidgets.QSpinBox(self.groupBox_7)
        self.sbNaNReplacement.setMinimum(-999999)
        self.sbNaNReplacement.setMaximum(9999997)
        self.sbNaNReplacement.setObjectName("sbNaNReplacement")
        self.horizontalLayout_5.addWidget(self.sbNaNReplacement)
        self.verticalLayout_10.addLayout(self.horizontalLayout_5)
        self.gridLayout_7.addLayout(self.verticalLayout_10, 0, 0, 1, 1)
        self.verticalLayout_2.addWidget(self.groupBox_7)
        self.groupBox = QtWidgets.QGroupBox(self.tImport)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_16 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_16.setObjectName("gridLayout_16")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.general_imp_delimiter = QtWidgets.QLineEdit(self.groupBox)
        self.general_imp_delimiter.setObjectName("general_imp_delimiter")
        self.gridLayout_3.addWidget(self.general_imp_delimiter, 5, 3, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 5, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.gridLayout_3.addWidget(self.label_7, 0, 3, 1, 1)
        self.general_imp_decimal_sep = QtWidgets.QLineEdit(self.groupBox)
        self.general_imp_decimal_sep.setObjectName("general_imp_decimal_sep")
        self.gridLayout_3.addWidget(self.general_imp_decimal_sep, 6, 3, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 6, 0, 1, 1)
        self.dx_imp_delimiter = QtWidgets.QLineEdit(self.groupBox)
        self.dx_imp_delimiter.setObjectName("dx_imp_delimiter")
        self.gridLayout_3.addWidget(self.dx_imp_delimiter, 5, 1, 1, 1)
        self.csv_imp_delimiter = QtWidgets.QLineEdit(self.groupBox)
        self.csv_imp_delimiter.setObjectName("csv_imp_delimiter")
        self.gridLayout_3.addWidget(self.csv_imp_delimiter, 5, 2, 1, 1)
        self.dx_imp_decimal_sep = QtWidgets.QLineEdit(self.groupBox)
        self.dx_imp_decimal_sep.setObjectName("dx_imp_decimal_sep")
        self.gridLayout_3.addWidget(self.dx_imp_decimal_sep, 6, 1, 1, 1)
        self.csv_imp_decimal_sep = QtWidgets.QLineEdit(self.groupBox)
        self.csv_imp_decimal_sep.setObjectName("csv_imp_decimal_sep")
        self.gridLayout_3.addWidget(self.csv_imp_decimal_sep, 6, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.groupBox)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 0, 2, 1, 1)
        self.verticalLayout_9.addLayout(self.gridLayout_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.groupBox_5 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.dx_import_spectra_name_from_filename = QtWidgets.QRadioButton(self.groupBox_5)
        self.dx_import_spectra_name_from_filename.setObjectName("dx_import_spectra_name_from_filename")
        self.verticalLayout_4.addWidget(self.dx_import_spectra_name_from_filename)
        self.rb_DX_import_from_title = QtWidgets.QRadioButton(self.groupBox_5)
        self.rb_DX_import_from_title.setChecked(True)
        self.rb_DX_import_from_title.setObjectName("rb_DX_import_from_title")
        self.verticalLayout_4.addWidget(self.rb_DX_import_from_title)
        self.dx_if_title_is_empty_use_filename = QtWidgets.QCheckBox(self.groupBox_5)
        self.dx_if_title_is_empty_use_filename.setObjectName("dx_if_title_is_empty_use_filename")
        self.verticalLayout_4.addWidget(self.dx_if_title_is_empty_use_filename)
        self.gridLayout_9.addLayout(self.verticalLayout_4, 0, 0, 1, 1)
        self.horizontalLayout_2.addWidget(self.groupBox_5)
        self.groupBox_6 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_6.setObjectName("groupBox_6")
        self.gridLayout_15 = QtWidgets.QGridLayout(self.groupBox_6)
        self.gridLayout_15.setObjectName("gridLayout_15")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.label = QtWidgets.QLabel(self.groupBox_6)
        self.label.setObjectName("label")
        self.verticalLayout_8.addWidget(self.label)
        self.general_import_spectra_name_from_filename = QtWidgets.QRadioButton(self.groupBox_6)
        self.general_import_spectra_name_from_filename.setObjectName("general_import_spectra_name_from_filename")
        self.verticalLayout_8.addWidget(self.general_import_spectra_name_from_filename)
        self.rb_general_import_from_header = QtWidgets.QRadioButton(self.groupBox_6)
        self.rb_general_import_from_header.setChecked(True)
        self.rb_general_import_from_header.setObjectName("rb_general_import_from_header")
        self.verticalLayout_8.addWidget(self.rb_general_import_from_header)
        self.general_if_header_is_empty_use_filename = QtWidgets.QCheckBox(self.groupBox_6)
        self.general_if_header_is_empty_use_filename.setObjectName("general_if_header_is_empty_use_filename")
        self.verticalLayout_8.addWidget(self.general_if_header_is_empty_use_filename)
        self.gridLayout_15.addLayout(self.verticalLayout_8, 0, 0, 1, 1)
        self.horizontalLayout_2.addWidget(self.groupBox_6)
        self.verticalLayout_9.addLayout(self.horizontalLayout_2)
        self.gridLayout_16.addLayout(self.verticalLayout_9, 1, 0, 1, 1)
        self.verticalLayout_2.addWidget(self.groupBox)
        self.groupBox_2 = QtWidgets.QGroupBox(self.tImport)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.excel_imp_as_text = QtWidgets.QCheckBox(self.groupBox_2)
        self.excel_imp_as_text.setObjectName("excel_imp_as_text")
        self.verticalLayout_3.addWidget(self.excel_imp_as_text)
        self.label_10 = QtWidgets.QLabel(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_10.sizePolicy().hasHeightForWidth())
        self.label_10.setSizePolicy(sizePolicy)
        self.label_10.setWordWrap(True)
        self.label_10.setObjectName("label_10")
        self.verticalLayout_3.addWidget(self.label_10)
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_8 = QtWidgets.QLabel(self.groupBox_2)
        self.label_8.setObjectName("label_8")
        self.gridLayout_4.addWidget(self.label_8, 0, 0, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.groupBox_2)
        self.label_9.setObjectName("label_9")
        self.gridLayout_4.addWidget(self.label_9, 1, 0, 1, 1)
        self.clip_imp_decimal_sep = QtWidgets.QLineEdit(self.groupBox_2)
        self.clip_imp_decimal_sep.setObjectName("clip_imp_decimal_sep")
        self.gridLayout_4.addWidget(self.clip_imp_decimal_sep, 1, 1, 1, 1)
        self.clip_imp_delimiter = QtWidgets.QLineEdit(self.groupBox_2)
        self.clip_imp_delimiter.setObjectName("clip_imp_delimiter")
        self.gridLayout_4.addWidget(self.clip_imp_delimiter, 0, 1, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout_4)
        self.gridLayout_10.addLayout(self.verticalLayout_3, 0, 0, 1, 1)
        self.verticalLayout_2.addWidget(self.groupBox_2)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tImport, "")
        self.tExport = QtWidgets.QWidget()
        self.tExport.setObjectName("tExport")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.tExport)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_4 = QtWidgets.QGroupBox(self.tExport)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_18 = QtWidgets.QLabel(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_18.sizePolicy().hasHeightForWidth())
        self.label_18.setSizePolicy(sizePolicy)
        self.label_18.setWordWrap(True)
        self.label_18.setObjectName("label_18")
        self.verticalLayout_5.addWidget(self.label_18)
        self.clip_exp_include_group_name = QtWidgets.QCheckBox(self.groupBox_4)
        self.clip_exp_include_group_name.setObjectName("clip_exp_include_group_name")
        self.verticalLayout_5.addWidget(self.clip_exp_include_group_name)
        self.clip_exp_include_header = QtWidgets.QCheckBox(self.groupBox_4)
        self.clip_exp_include_header.setObjectName("clip_exp_include_header")
        self.verticalLayout_5.addWidget(self.clip_exp_include_header)
        self.gridLayout_6 = QtWidgets.QGridLayout()
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.label_19 = QtWidgets.QLabel(self.groupBox_4)
        self.label_19.setObjectName("label_19")
        self.gridLayout_6.addWidget(self.label_19, 0, 0, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.groupBox_4)
        self.label_20.setObjectName("label_20")
        self.gridLayout_6.addWidget(self.label_20, 1, 0, 1, 1)
        self.clip_exp_decimal_sep = QtWidgets.QLineEdit(self.groupBox_4)
        self.clip_exp_decimal_sep.setObjectName("clip_exp_decimal_sep")
        self.gridLayout_6.addWidget(self.clip_exp_decimal_sep, 1, 1, 1, 1)
        self.clip_exp_delimiter = QtWidgets.QLineEdit(self.groupBox_4)
        self.clip_exp_delimiter.setObjectName("clip_exp_delimiter")
        self.gridLayout_6.addWidget(self.clip_exp_delimiter, 0, 1, 1, 1)
        self.verticalLayout_5.addLayout(self.gridLayout_6)
        self.gridLayout_12.addLayout(self.verticalLayout_5, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_4, 1, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tExport)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.files_exp_include_group_name = QtWidgets.QCheckBox(self.groupBox_3)
        self.files_exp_include_group_name.setObjectName("files_exp_include_group_name")
        self.verticalLayout_6.addWidget(self.files_exp_include_group_name)
        self.files_exp_include_header = QtWidgets.QCheckBox(self.groupBox_3)
        self.files_exp_include_header.setObjectName("files_exp_include_header")
        self.verticalLayout_6.addWidget(self.files_exp_include_header)
        self.gridLayout_11.addLayout(self.verticalLayout_6, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_3, 0, 0, 1, 1)
        self.gridLayout_8.addLayout(self.gridLayout, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tExport, "")
        self.tPlotting = QtWidgets.QWidget()
        self.tPlotting.setObjectName("tPlotting")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.tPlotting)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.groupBox_8 = QtWidgets.QGroupBox(self.tPlotting)
        self.groupBox_8.setObjectName("groupBox_8")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.groupBox_8)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.same_color_in_group = QtWidgets.QCheckBox(self.groupBox_8)
        self.same_color_in_group.setObjectName("same_color_in_group")
        self.verticalLayout_11.addWidget(self.same_color_in_group)
        self.different_line_style_among_groups = QtWidgets.QCheckBox(self.groupBox_8)
        self.different_line_style_among_groups.setObjectName("different_line_style_among_groups")
        self.verticalLayout_11.addWidget(self.different_line_style_among_groups)
        self.groupBox_9 = QtWidgets.QGroupBox(self.groupBox_8)
        self.groupBox_9.setObjectName("groupBox_9")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.groupBox_9)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.rbDefaultColorScheme = QtWidgets.QRadioButton(self.groupBox_9)
        self.rbDefaultColorScheme.setObjectName("rbDefaultColorScheme")
        self.verticalLayout_7.addWidget(self.rbDefaultColorScheme)
        self.rbHSVColorScheme = QtWidgets.QRadioButton(self.groupBox_9)
        self.rbHSVColorScheme.setObjectName("rbHSVColorScheme")
        self.verticalLayout_7.addWidget(self.rbHSVColorScheme)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_32 = QtWidgets.QLabel(self.groupBox_9)
        self.label_32.setObjectName("label_32")
        self.horizontalLayout_4.addWidget(self.label_32)
        self.sbHues = QtWidgets.QSpinBox(self.groupBox_9)
        self.sbHues.setMinimum(1)
        self.sbHues.setMaximum(999999)
        self.sbHues.setObjectName("sbHues")
        self.horizontalLayout_4.addWidget(self.sbHues)
        self.verticalLayout_7.addLayout(self.horizontalLayout_4)
        self.gridLayout_14 = QtWidgets.QGridLayout()
        self.gridLayout_14.setObjectName("gridLayout_14")
        self.label_27 = QtWidgets.QLabel(self.groupBox_9)
        self.label_27.setObjectName("label_27")
        self.gridLayout_14.addWidget(self.label_27, 0, 2, 1, 1)
        self.sbMinHue = QtWidgets.QSpinBox(self.groupBox_9)
        self.sbMinHue.setMinimum(0)
        self.sbMinHue.setMaximum(360)
        self.sbMinHue.setObjectName("sbMinHue")
        self.gridLayout_14.addWidget(self.sbMinHue, 0, 3, 1, 1)
        self.label_28 = QtWidgets.QLabel(self.groupBox_9)
        self.label_28.setObjectName("label_28")
        self.gridLayout_14.addWidget(self.label_28, 0, 4, 1, 1)
        self.sbMaxHue = QtWidgets.QSpinBox(self.groupBox_9)
        self.sbMaxHue.setMinimum(0)
        self.sbMaxHue.setMaximum(360)
        self.sbMaxHue.setObjectName("sbMaxHue")
        self.gridLayout_14.addWidget(self.sbMaxHue, 0, 5, 1, 1)
        self.label_26 = QtWidgets.QLabel(self.groupBox_9)
        self.label_26.setObjectName("label_26")
        self.gridLayout_14.addWidget(self.label_26, 1, 0, 1, 1)
        self.sbValues = QtWidgets.QSpinBox(self.groupBox_9)
        self.sbValues.setMinimum(1)
        self.sbValues.setMaximum(999999)
        self.sbValues.setObjectName("sbValues")
        self.gridLayout_14.addWidget(self.sbValues, 1, 1, 1, 1)
        self.label_29 = QtWidgets.QLabel(self.groupBox_9)
        self.label_29.setObjectName("label_29")
        self.gridLayout_14.addWidget(self.label_29, 1, 2, 1, 1)
        self.sbMinValue = QtWidgets.QSpinBox(self.groupBox_9)
        self.sbMinValue.setMinimum(0)
        self.sbMinValue.setMaximum(255)
        self.sbMinValue.setObjectName("sbMinValue")
        self.gridLayout_14.addWidget(self.sbMinValue, 1, 3, 1, 1)
        self.label_30 = QtWidgets.QLabel(self.groupBox_9)
        self.label_30.setObjectName("label_30")
        self.gridLayout_14.addWidget(self.label_30, 1, 4, 1, 1)
        self.sbMaxValue = QtWidgets.QSpinBox(self.groupBox_9)
        self.sbMaxValue.setMinimum(0)
        self.sbMaxValue.setMaximum(255)
        self.sbMaxValue.setObjectName("sbMaxValue")
        self.gridLayout_14.addWidget(self.sbMaxValue, 1, 5, 1, 1)
        self.verticalLayout_7.addLayout(self.gridLayout_14)
        self.rbUserColorScheme = QtWidgets.QRadioButton(self.groupBox_9)
        self.rbUserColorScheme.setObjectName("rbUserColorScheme")
        self.verticalLayout_7.addWidget(self.rbUserColorScheme)
        self.txbUserColorScheme = QtWidgets.QPlainTextEdit(self.groupBox_9)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txbUserColorScheme.sizePolicy().hasHeightForWidth())
        self.txbUserColorScheme.setSizePolicy(sizePolicy)
        self.txbUserColorScheme.setMinimumSize(QtCore.QSize(0, 0))
        self.txbUserColorScheme.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.txbUserColorScheme.setObjectName("txbUserColorScheme")
        self.verticalLayout_7.addWidget(self.txbUserColorScheme)
        self.cbHSVReversed = QtWidgets.QCheckBox(self.groupBox_9)
        self.cbHSVReversed.setObjectName("cbHSVReversed")
        self.verticalLayout_7.addWidget(self.cbHSVReversed)
        self.verticalLayout_11.addWidget(self.groupBox_9)
        self.verticalLayout_12.addWidget(self.groupBox_8)
        self.antialiasing = QtWidgets.QCheckBox(self.tPlotting)
        self.antialiasing.setObjectName("antialiasing")
        self.verticalLayout_12.addWidget(self.antialiasing)
        self.show_grid = QtWidgets.QCheckBox(self.tPlotting)
        self.show_grid.setObjectName("show_grid")
        self.verticalLayout_12.addWidget(self.show_grid)
        self.chbReverseZOrder = QtWidgets.QCheckBox(self.tPlotting)
        self.chbReverseZOrder.setObjectName("chbReverseZOrder")
        self.verticalLayout_12.addWidget(self.chbReverseZOrder)
        self.gridLayout_13 = QtWidgets.QGridLayout()
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.label_23 = QtWidgets.QLabel(self.tPlotting)
        self.label_23.setObjectName("label_23")
        self.gridLayout_13.addWidget(self.label_23, 4, 0, 1, 1)
        self.label_31 = QtWidgets.QLabel(self.tPlotting)
        self.label_31.setObjectName("label_31")
        self.gridLayout_13.addWidget(self.label_31, 0, 0, 1, 1)
        self.label_21 = QtWidgets.QLabel(self.tPlotting)
        self.label_21.setObjectName("label_21")
        self.gridLayout_13.addWidget(self.label_21, 2, 0, 1, 1)
        self.left_axis_label = QtWidgets.QLineEdit(self.tPlotting)
        self.left_axis_label.setObjectName("left_axis_label")
        self.gridLayout_13.addWidget(self.left_axis_label, 2, 2, 1, 1)
        self.line_width = QtWidgets.QDoubleSpinBox(self.tPlotting)
        self.line_width.setDecimals(1)
        self.line_width.setMinimum(0.1)
        self.line_width.setMaximum(50.0)
        self.line_width.setProperty("value", 1.0)
        self.line_width.setObjectName("line_width")
        self.gridLayout_13.addWidget(self.line_width, 4, 2, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.tPlotting)
        self.label_14.setObjectName("label_14")
        self.gridLayout_13.addWidget(self.label_14, 3, 0, 1, 1)
        self.legend_spacing = QtWidgets.QSpinBox(self.tPlotting)
        self.legend_spacing.setMaximum(200)
        self.legend_spacing.setSingleStep(1)
        self.legend_spacing.setObjectName("legend_spacing")
        self.gridLayout_13.addWidget(self.legend_spacing, 5, 2, 1, 1)
        self.label_22 = QtWidgets.QLabel(self.tPlotting)
        self.label_22.setObjectName("label_22")
        self.gridLayout_13.addWidget(self.label_22, 1, 0, 1, 1)
        self.label_24 = QtWidgets.QLabel(self.tPlotting)
        self.label_24.setObjectName("label_24")
        self.gridLayout_13.addWidget(self.label_24, 5, 0, 1, 1)
        self.bottom_axis_label = QtWidgets.QLineEdit(self.tPlotting)
        self.bottom_axis_label.setObjectName("bottom_axis_label")
        self.gridLayout_13.addWidget(self.bottom_axis_label, 3, 2, 1, 1)
        self.graph_title = QtWidgets.QLineEdit(self.tPlotting)
        self.graph_title.setObjectName("graph_title")
        self.gridLayout_13.addWidget(self.graph_title, 1, 2, 1, 1)
        self.grid_alpha = QtWidgets.QDoubleSpinBox(self.tPlotting)
        self.grid_alpha.setDecimals(1)
        self.grid_alpha.setMinimum(0.0)
        self.grid_alpha.setMaximum(1.0)
        self.grid_alpha.setSingleStep(0.1)
        self.grid_alpha.setProperty("value", 0.3)
        self.grid_alpha.setObjectName("grid_alpha")
        self.gridLayout_13.addWidget(self.grid_alpha, 0, 2, 1, 1)
        self.verticalLayout_12.addLayout(self.gridLayout_13)
        self.tabWidget.addTab(self.tPlotting, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnRestoreDefaultSettings = QtWidgets.QPushButton(Dialog)
        self.btnRestoreDefaultSettings.setMinimumSize(QtCore.QSize(170, 0))
        self.btnRestoreDefaultSettings.setObjectName("btnRestoreDefaultSettings")
        self.horizontalLayout.addWidget(self.btnRestoreDefaultSettings)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.horizontalLayout.addWidget(self.buttonBox)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout_5.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.tabWidget.setCurrentIndex(0)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)
        Dialog.setTabOrder(self.tabWidget, self.general_imp_delimiter)
        Dialog.setTabOrder(self.general_imp_delimiter, self.general_imp_decimal_sep)
        Dialog.setTabOrder(self.general_imp_decimal_sep, self.excel_imp_as_text)
        Dialog.setTabOrder(self.excel_imp_as_text, self.clip_imp_delimiter)
        Dialog.setTabOrder(self.clip_imp_delimiter, self.clip_imp_decimal_sep)
        Dialog.setTabOrder(self.clip_imp_decimal_sep, self.clip_exp_include_group_name)
        Dialog.setTabOrder(self.clip_exp_include_group_name, self.clip_exp_include_header)
        Dialog.setTabOrder(self.clip_exp_include_header, self.clip_exp_delimiter)
        Dialog.setTabOrder(self.clip_exp_delimiter, self.clip_exp_decimal_sep)
        Dialog.setTabOrder(self.clip_exp_decimal_sep, self.left_axis_label)
        Dialog.setTabOrder(self.left_axis_label, self.bottom_axis_label)
        Dialog.setTabOrder(self.bottom_axis_label, self.btnRestoreDefaultSettings)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label_11.setText(_translate("Dialog", "\\n - new line, \\r - cartrige return, \\t - tabulator\n"
"If delimiter field is empty, lines will be splitted by any whitespace (\' \', \'\\n\', \'\\r\', \'\\t\', \'\\f\', \'\\v\') and empty strings will be discarded (empty entries will be always removed). Note, that this does not apply for CSV file, if delimiter for CSV file is empty, default comma \',\' will be used."))
        self.groupBox_7.setTitle(_translate("Dialog", "CSV and Other files + Clipboard settings"))
        self.cbRemoveEmptyEntries.setText(_translate("Dialog", "Remove empty entries for each parsed line"))
        self.label_12.setText(_translate("Dialog", "Skip first n columns:"))
        self.cbSkipNaNColumns.setText(_translate("Dialog", "Skip columns that contain NaN (Not a Number) values"))
        self.label_13.setText(_translate("Dialog", "Replacement for NaN values:"))
        self.groupBox.setTitle(_translate("Dialog", "Files"))
        self.label_3.setText(_translate("Dialog", "Delimiter"))
        self.label_7.setText(_translate("Dialog", "Other (*.txt, etc.)"))
        self.label_4.setText(_translate("Dialog", "Decimal separator"))
        self.label_6.setText(_translate("Dialog", "DX file"))
        self.label_5.setText(_translate("Dialog", "CSV file"))
        self.groupBox_5.setTitle(_translate("Dialog", "DX file"))
        self.dx_import_spectra_name_from_filename.setText(_translate("Dialog", "Import spectra name\n"
"from filename"))
        self.rb_DX_import_from_title.setText(_translate("Dialog", "Import spectra name\n"
"from ##TITLE entry"))
        self.dx_if_title_is_empty_use_filename.setText(_translate("Dialog", "If ##TITLE is empty, use\n"
"spectra name from filename"))
        self.groupBox_6.setTitle(_translate("Dialog", "CSV and Other files"))
        self.label.setText(_translate("Dialog", "If possible (for non-concatenated data),"))
        self.general_import_spectra_name_from_filename.setText(_translate("Dialog", "Import spectra name\n"
"from filename"))
        self.rb_general_import_from_header.setText(_translate("Dialog", "Import spectra name\n"
"from header"))
        self.general_if_header_is_empty_use_filename.setText(_translate("Dialog", "If header is empty, use\n"
"spectra name from filename"))
        self.groupBox_2.setTitle(_translate("Dialog", "Clipboard"))
        self.excel_imp_as_text.setText(_translate("Dialog", "Import data from MS Excel as text (if checked, decimal precision will be lost)"))
        self.label_10.setText(_translate("Dialog", "Set how to import text data from clipboard (in order to retain compatibility with MS Excel/Origin, keep delimiter set as tabulator \\t or empty (any whitespace characters), decimal separator is application specific)"))
        self.label_8.setText(_translate("Dialog", "Delimiter"))
        self.label_9.setText(_translate("Dialog", "Decimal separator"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tImport), _translate("Dialog", "Import"))
        self.groupBox_4.setTitle(_translate("Dialog", "Clipboard"))
        self.label_18.setText(_translate("Dialog", "Set how to export text data to clipboard (in order to retain compatibility with MS Excel/Origin, keep delimiter set as tabulator \\t, decimal separator is application specific)"))
        self.clip_exp_include_group_name.setText(_translate("Dialog", "Include group name"))
        self.clip_exp_include_header.setText(_translate("Dialog", "Include header"))
        self.label_19.setText(_translate("Dialog", "Delimiter"))
        self.label_20.setText(_translate("Dialog", "Decimal separator"))
        self.groupBox_3.setTitle(_translate("Dialog", "Files"))
        self.files_exp_include_group_name.setText(_translate("Dialog", "Include group name"))
        self.files_exp_include_header.setText(_translate("Dialog", "Include header"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tExport), _translate("Dialog", "Export"))
        self.groupBox_8.setTitle(_translate("Dialog", "Color and line style"))
        self.same_color_in_group.setText(_translate("Dialog", "Plot spectra with same color in groups"))
        self.different_line_style_among_groups.setText(_translate("Dialog", "Plot spectra with different line style among groups"))
        self.groupBox_9.setTitle(_translate("Dialog", "Color schemes"))
        self.rbDefaultColorScheme.setText(_translate("Dialog", "Default (red, green, blue, black, yellow, magenta, cyan, gray, .... and repeat)"))
        self.rbHSVColorScheme.setText(_translate("Dialog", "HSV/HSB gradient"))
        self.label_32.setText(_translate("Dialog", "Number of spectra (for gradients)"))
        self.label_27.setText(_translate("Dialog", "Min hue"))
        self.label_28.setText(_translate("Dialog", "Max hue"))
        self.label_26.setText(_translate("Dialog", "Values / \n"
"Brightnesses"))
        self.label_29.setText(_translate("Dialog", "Min value"))
        self.label_30.setText(_translate("Dialog", "Max value"))
        self.rbUserColorScheme.setText(_translate("Dialog", "User defined gradient (1st column position - 0->1, next RGBA values - 0->1 each)"))
        self.cbHSVReversed.setText(_translate("Dialog", "Reversed color scheme"))
        self.antialiasing.setText(_translate("Dialog", "Antialiasing"))
        self.show_grid.setText(_translate("Dialog", "Show Grid"))
        self.chbReverseZOrder.setText(_translate("Dialog", "Reverse Z order (first spectra will be ploted behind next ones)"))
        self.label_23.setText(_translate("Dialog", "Line width"))
        self.label_31.setText(_translate("Dialog", "Grid alpha"))
        self.label_21.setText(_translate("Dialog", "Left axis label"))
        self.label_14.setText(_translate("Dialog", "Bottom axis label"))
        self.label_22.setText(_translate("Dialog", "Graph title"))
        self.label_24.setText(_translate("Dialog", "Legend spacng"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tPlotting), _translate("Dialog", "Plotting"))
        self.btnRestoreDefaultSettings.setText(_translate("Dialog", "Restore Default Settings"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




from PyQt5.QtWidgets import QLineEdit, QWidget, QHBoxLayout, QApplication, QToolButton
from PyQt5.QtGui import QRegion, QMouseEvent, QWheelEvent
from PyQt5.QtCore import Qt, pyqtSignal

import numpy as np


def decor_t2f(func):
    def text2float():
        try:
            val = float(func())
        except ValueError:
            return
        return val
    return text2float


class DialLineEdit(QHBoxLayout):

    # check_changed = pyqtSignal(list)  # list of abbreviations of checked items
    simulation_requested = pyqtSignal()

    def __init__(self, text: str = '', lb_val=lambda: 0, ub_val=lambda: np.inf, parent=None):
        """items is dictionary that uses short names as keys and longer description as values"""
        super(DialLineEdit, self).__init__(parent)

        self.lb_val = decor_t2f(lb_val)  # lower bound func
        self.ub_val = decor_t2f(ub_val)  # upper bound func

        self.le = _DialLineEdit(text, self)
        self.dial = RoundButton(self.le, self)
        self.addWidget(self.le)
        self.addWidget(self.dial)

    def setVisible(self, visible):
        self.le.setVisible(visible)
        self.dial.setVisible(visible)

    def setEnabled(self, enabled: bool):
        self.le.setEnabled(enabled)
        self.dial.setEnabled(enabled)
        super(DialLineEdit, self).setEnabled(enabled)

    def __getattr__(self, item):  # redirect to main QLineEdit
        return self.le.__getattribute__(item)


class _DialLineEdit(QLineEdit):

    def __init__(self, text: str = '', parent=None):
        super(_DialLineEdit, self).__init__(text, None)
        self.lb_val = parent.lb_val  # lower bound func
        self.ub_val = parent.ub_val  # upper bound func
        self.simulation_requested = parent.simulation_requested
        self.returnPressed.connect(lambda: self.simulation_requested.emit())  # simulate when enter was pressed

    def wheelEvent(self, e: QWheelEvent):
        super(_DialLineEdit, self).wheelEvent(e)
        turns = e.angleDelta().y() // 120

        try:
            val = float(self.text())
        except ValueError:
            return

        lb = self.lb_val()
        ub = self.ub_val()

        if lb is None or ub is None:
            return

        coef = turns / 10 if turns > 0 else turns / 11

        val += val * np.sign(val) * coef  # increase/decrease in 10th of original value

        if val < lb:
            val = lb

        if val > ub:
            val = ub

        self.setText(f'{val:.4g}')
        self.simulation_requested.emit()


class RoundButton(QToolButton):
    def __init__(self, lineedit, parent=None):
        super(RoundButton, self).__init__(None)

        self.lb_val = parent.lb_val  # lower bound func
        self.ub_val = parent.ub_val  # upper bound func
        self.simulation_requested = parent.simulation_requested

        # self.left_btn_pressed = False
        self.init_val = None
        self.initial_y_pos = 0
        self.le = lineedit

        stylesheet = """
        RoundButton {
            background-color : darkgreen;
        }
        
        RoundButton::hover {
            background-color : darkred;
        }
        
        RoundButton::pressed {
            background-color : red;
        }
        """
        self.setStyleSheet(stylesheet)

    def mouseMoveEvent(self, e: QMouseEvent):
        super(RoundButton, self).mouseMoveEvent(e)

        if self.init_val is None:
            return

        lb = self.lb_val()
        ub = self.ub_val()

        if lb is None or ub is None:
            return

        dy = self.initial_y_pos - e.y()

        cb = ub if dy >= 0 else lb  # current bound

        if np.abs(cb) != np.inf:
            exp_term = (1 - np.exp(-np.abs(dy * 1e-2)))
            diff = cb - self.init_val

            val = self.init_val + diff * exp_term
        else:
            val = self.init_val * (1 + np.sign(self.init_val) * dy * 1e-2)

        if val < lb:
            val = lb

        if val > ub:
            val = ub

        self.le.setText(f'{val:.4g}')
        self.simulation_requested.emit()

    def mousePressEvent(self, e: QMouseEvent):
        try:
            val = float(self.le.text())
        except ValueError:
            return

        self.init_val = val
        self.initial_y_pos = e.y()
        super(RoundButton, self).mousePressEvent(e)

    def mouseReleaseEvent(self, e: QMouseEvent):
        self.init_val = None
        super(RoundButton, self).mouseReleaseEvent(e)

    def resizeEvent(self, e):
        r = self.rect()
        x, y, w, h = r.x(), r.y(), r.width(), r.height()
        x += w // 4
        y += h // 4
        w //= 2
        h //= 2
        self.setMask(QRegion(x, y, w, h, QRegion.Ellipse))
        super(RoundButton, self).resizeEvent(e)


if __name__ == '__main__':
    import sys

    app = QApplication(sys.argv)
    w = QWidget(None)
    w.setLayout(DialLineEdit('10'))
    # cb = DialLineEdit('10')
    w.show()
    sys.exit(app.exec_())



from PyQt5.QtGui import QBrush, QColor
from PyQt5.QtWidgets import QMessageBox
from .genericinputwidget import GenericInputWidget
from .mylineedit import MyLineEdit
from PyQt5.QtCore import Qt

from ..plotwidget import PlotWidget
import pyqtgraph as pg


class RangeWidget(GenericInputWidget):

    def __init__(self, dock_widget, accepted_func=None, label_text='Set xrange', title='SetRangeDialog', parent=None):

        if RangeWidget.instance is not None:
            PlotWidget._instance.removeItem(RangeWidget.instance.lr)

        self.dock_widget = dock_widget
        self.accepted_func = accepted_func

        self.le_x0 = MyLineEdit()
        self.le_x1 = MyLineEdit()

        super(RangeWidget, self).__init__([('x0', self.le_x0), ('x1', self.le_x1)],
                                          label_text=label_text,
                                          title=title, parent=parent)

        self.lr = PlotWidget.add_linear_region(z_value=1e8)
        self.lr.sigRegionChanged.connect(lambda: self.update_values())

        #
        # f = 0.87
        # # x0, x1 = x0_value, x1_value
        # x0, x1 = PlotWidget._instance.getViewBox().viewRange()[0]
        # xy_dif = x1 - x0
        # self.lr = pg.LinearRegionItem([x0 + (1 - f) * xy_dif, x0 + f * xy_dif],
        #                               brush=QBrush(QColor(0, 255, 0, 20)))
        #
        # self.lr.setZValue(1e6)
        # PlotWidget._instance.addItem(self.lr)

        self.le_x0.focus_lost.connect(self.update_region)
        self.le_x1.focus_lost.connect(self.update_region)
        self.le_x0.returnPressed.connect(self.update_region)
        self.le_x1.returnPressed.connect(self.update_region)

        self.lr.sigRegionChanged.connect(self.update_values)
        self.update_values()
        # we have to update region, otherwide there would be some bug in with manually mooving the region
        self.update_region()

        self.returned_range = None

        self.dock_widget.parent().resizeDocks([self.dock_widget], [250], Qt.Vertical)
        self.dock_widget.titleBarWidget().setText(title)
        self.dock_widget.setWidget(self)
        self.dock_widget.setVisible(True)

        # set focus
        self.le_x0.setFocus(Qt.TabFocusReason)

    def update_values(self):
        x0, x1 = self.lr.getRegion()
        self.le_x0.setText("{:.4g}".format(x0))
        self.le_x1.setText("{:.4g}".format(x1))

    # def update_values(self):
    #     x0, x1 = self.lr.getRegion()
    #     self.le_x0.setText("{:.4g}".format(x0))
    #     self.le_x1.setText("{:.4g}".format(x1))

    def update_region(self):
        try:
            x0, x1 = float(self.le_x0.text()), float(self.le_x1.text())
            if x0 <= x1:
                self.lr.setRegion((x0, x1))
        except ValueError:
            pass

    def accept(self):
        try:
            self.returned_range = [float(self.le_x0.text()), float(self.le_x1.text())]

            # swap range if it is revesed
            if self.returned_range[0] > self.returned_range[1]:
                temp = self.returned_range[0]
                self.returned_range[0] = self.returned_range[1]
                self.returned_range[1] = temp

        except ValueError:
            QMessageBox.warning(self, "Warning", "Invalid format of the range.", QMessageBox.Ok)
            return

        PlotWidget.remove_linear_region()
        self.dock_widget.setVisible(False)
        super(RangeWidget, self).accept()

        self.accepted_func()

    def reject(self):
        PlotWidget.remove_linear_region()
        self.dock_widget.setVisible(False)
        super(RangeWidget, self).reject()


# if __name__ == "__main__":
    # import sys
    # from PyQt5.QtWidgets import QApplication
    #
    # app = QApplication(sys.argv)
    # Dialog = RangeWidget()
    # Dialog.show()
    # sys.exit(app.exec_())



from PyQt5.QtCore import Qt
from spectramanipulator.spectrum import Spectrum, SpectrumList

import numpy as np

from copy import deepcopy


class GenericItem:

    # static signals that are instantiated after formation of TreeView and root
    data_modified_signal = None
    info_modified_signal = None

    def __init__(self, name, info, parent=None):

        self.check_state = Qt.Unchecked

        self.name = str(name)
        self.info = str(info)

        self.parent = parent
        self.setParent(parent)

        self.children = []

    def is_root(self):
        return self.parent is None

    def isChecked(self):
        return self.check_state == Qt.Checked or self.check_state == Qt.PartiallyChecked

    def setParent(self, parent, row=None):
        if parent is not None:
            self.parent = parent
            self.parent.appendChild(self, row)

    def appendChild(self, child, row=None):
        self.children.insert(row if row is not None else len(self.children), child)
        # self.children.append(child)

    def childAtRow(self, row):
        try:
            return self.children[row]
        except IndexError:
            # print("index error")
            return

    def move_child(self, row_of_child: int, to_row=0):
        self.children.insert(to_row, self.children.pop(row_of_child))

    def rowOfChild(self, child):
        for i, item in enumerate(self.children):
            if item == child:
                return i
        return -1

    def removeChildAtRow(self, row):
        # try:

        del self.children[row]

        # value = self.children[row]
        # self.children.remove(value)
        # except:
        #     pass

        return True

    def row(self):
        if self.parent is not None:
            return self.parent.rowOfChild(self)

    def removeChild(self, child):
        self.children.remove(child)

    def root(self):
        parent = self.parent
        if parent is None:
            return self
        while parent.parent is not None:
            parent = parent.parent
        return parent

    def __len__(self):
        return len(self.children)

    def __getitem__(self, item):
        return self.children[item]

    def __iter__(self):
        return iter(self.children)

    # def add_to_list(self, spectra=None):
    #     if self.__class__ == SpectrumItem or self.__class__ == SpectrumItemGroup:
    #         un.add_to_list(self if spectra is None else spectra)
    #         # print('add_to_list - generic')

    def _redraw_all_spectra(self):
        items = [self] if len(self.children) == 0 else self.children
        if GenericItem.data_modified_signal is not None:
            GenericItem.data_modified_signal.emit(items)  # call static function

    def _update_view(self):
        items = [self] if len(self.children) == 0 else self.children
        if GenericItem.info_modified_signal is not None:
            GenericItem.info_modified_signal.emit(items)  # call static function


class SpectrumItem(GenericItem, Spectrum):

    def __init__(self, name='', info='', parent=None, color=None, line_width=None, line_alpha=255,
                 line_type=None, symbol=None, symbol_brush=None, sym_brush_alpha=255, symbol_fill=None,
                 sym_fill_alpha=255, symbol_size=8, plot_legend=True, data=None, filepath=None,
                 assume_sorted=False):

        Spectrum.__init__(self, data, name=name, filepath=filepath, assume_sorted=assume_sorted)

        super(SpectrumItem, self).__init__(name, info, parent)

        self.color = color  # line color
        self.line_width = line_width
        self.line_type = line_type
        self.plot_legend = plot_legend

        self.symbol = symbol
        self.symbol_brush = symbol_brush
        self.symbol_fill = symbol_fill
        self.symbol_size = symbol_size

        self.line_alpha = line_alpha
        self.sym_brush_alpha = sym_brush_alpha
        self.sym_fill_alpha = sym_fill_alpha

    @classmethod
    def from_spectrum(cls, spectrum, info='', parent=None, copy_spectrum=False):
        si = cls(spectrum.name, info, parent,  # for backward compatibility
                 color=getattr(spectrum, 'color', None),
                 line_width=getattr(spectrum, 'line_width', None),
                 line_alpha=getattr(spectrum, 'line_alpha', 255),
                 line_type=getattr(spectrum, 'line_type', None),
                 symbol=getattr(spectrum, 'symbol', None),
                 symbol_brush=getattr(spectrum, 'symbol_brush', None),
                 sym_brush_alpha=getattr(spectrum, 'sym_brush_alpha', 255),
                 symbol_fill=getattr(spectrum, 'symbol_fill', None),
                 sym_fill_alpha=getattr(spectrum, 'sym_fill_alpha', 255),
                 symbol_size=getattr(spectrum, 'symbol_size', 8),
                 plot_legend=getattr(spectrum, 'plot_legend', True),
                 data=spectrum.data.copy() if copy_spectrum else spectrum.data,
                 filepath=spectrum.filepath,
                 assume_sorted=True)

        return si

    @classmethod
    def from_xy_values(cls, x_values, y_values, **kwargs):

        sp = super(SpectrumItem, cls).from_xy_values(x_values, y_values, **kwargs)

        return sp

    @property
    def x(self):
        return self.data[:, 0]

    @x.setter
    def x(self, array):
        self.data[:, 0] = array
        self._redraw_all_spectra()
        self._update_view()

    @property
    def y(self):
        return self.data[:, 1]

    @y.setter
    def y(self, array):
        self.data[:, 1] = array
        self._redraw_all_spectra()
        self._update_view()

    def spacing(self):
        """
        Returns a type and spacing of the x values:
            * r - probably regular spacing for all points
            * i - probably irregular spacing for all points

        The r/i distinction is made by checking differences between first and the end x values.
        Value is computed as average spacing:  x_max - x_min / number of points

        :return: str
        """

        spacing = (self.data[-1, 0] - self.data[0, 0]) / (self.data.shape[0] - 1)  # average spacing
        s_type = 'r'  # regular spacing
        if self.data.shape[0] > 2:
            # irregular spacing
            x_diff = self.data[1:, 0] - self.data[:-1, 0]  # x differences
            if not np.allclose(x_diff, x_diff[0]):
                s_type = 'i'

        return s_type + "{:.3g}".format(spacing)

    def set_style(self, color=None, line_width=None, line_type=None, redraw_spectra=True):
        """
        Set color, line width and line type of plotted spectrum.

        Parameters
        ----------
        color : {str, tuple}, optional
            Color of the spectrum, use string for common colors (eg. 'black', 'blue', 'red', ...)
            or tuple - red, green, blue, alpha components from 0 - 255, eg. (255, 0, 0, 255).
            If None (default), user defined color scheme will be used.
        line_width : {int, float}, optional
            Sets the line width of the plotted line. If None (default), user defined color scheme will be used.
        line_type : int 0-6, optional
            Sets the line type of the plotted line. See https://doc.qt.io/archives/qt-4.8/qt.html#PenStyle-enum
            for line types. If None (default), user defined color scheme will be used.
        redraw_spectra : bool
            If True (default), spectra will be redrawn.
        """
        # if color is not None:
        self.color = color
        # if line_width is not None:
        self.line_width = line_width
        # if line_type is not None:
        self.line_type = line_type
        if redraw_spectra:
            # from user_namespace import redraw_all_spectra
            self._redraw_all_spectra()

    def set_default_style(self):
        """
        Sets `color`, `line_width` and `line_type` to None. User defined color scheme will be used.

        Parameters
        ----------
        redraw_spectra : bool
            If True (default), spectra will be redrawn.
        """
        self.color = None
        self.line_width = None
        self.line_type = None

        self._redraw_all_spectra()

    def is_in_group(self):
        return not self.parent.is_root()

    def is_top_level(self):
        return self.parent.is_root()

    def __copy__(self):
        return deepcopy(self)


class SpectrumItemGroup(GenericItem, SpectrumList):

    def __init__(self, name='', info='', parent=None):
        super(SpectrumItemGroup, self).__init__(name, info, parent)

        # self.setup_fcn()

    def set_names(self, names):
        # super(SpectrumItemGroup, self).set_names(names)
        for sp, new_name in zip(self, names):
            sp.name = str(new_name)
        self._update_view()
        self._redraw_all_spectra()

    def set_plot_legend(self, plot_legend=True):
        """
        Sets whether to plot legend for this group or not and redraws all spectra.

        Parameters
        ----------
        plot_legend : bool
            Default True.
        """

        for sp in self:
            sp.set_plot_legend(plot_legend, False)

    def set_style(self, color=None, line_width=None, line_type=None):
        """
        Sets color, line width and line type of all group and redraws all spectra.

        Parameters
        ----------
        color : {str, tuple}, optional
            Color of the spectrum, use string for common colors (eg. 'black', 'blue', 'red', ...)
            or tuple - red, green, blue, alpha components from 0 - 255, eg. (255, 0, 0, 255).
            If None (default), user defined color scheme will be used.
        line_width : {int, float}, optional
            Sets the line width of the plotted line. If None (default), user defined color scheme will be used.
        line_type : int 0-6, optional
            Sets the line type of the plotted line. See https://doc.qt.io/archives/qt-4.8/qt.html#PenStyle-enum
            for line types. If None (default), user defined color scheme will be used.
        """

        for sp in self:
            sp.set_style(color, line_width, line_type, False)

        self._redraw_all_spectra()

    def is_top_level(self):
        return True
